@@ -19,10 +19,11 @@ class Argo:
         pass
 
     def __enter__(self):
+        print("trying to enter")
         self.conn = BlitzGateway(
             host=self.host, username=self.username, passwd=self.password
         )
-        self.conn.connect()
+        print("Connection successful: ", self.conn.connect())
         return self
 
     def __exit__(self, *exc):
