@@ -8,14 +8,16 @@ from typing import List
 from pathlib import Path
 import traceback
 
-import itertools
+from itertools import groupby
+import h5py
 import yaml
 from tqdm import tqdm
 from time import perf_counter
+from pathos.multiprocessing import Pool
 
 import numpy as np
 import pandas as pd
-from pathos.multiprocessing import Pool
+from scipy import ndimage
 
 from aliby.experiment import MetaData
 from aliby.haystack import initialise_tf
@@ -28,15 +30,26 @@ from agora.io.signal import Signal
 from extraction.core.extractor import Extractor, ExtractorParameters
 from extraction.core.functions.defaults import exparams_from_meta
 from postprocessor.core.processor import PostProcessor, PostProcessorParameters
+from postprocessor.compiler import ExperimentCompiler, PageOrganiser
+
+logging.basicConfig(
+    filename="aliby.log",
+    filemode="w",
+    format="%(name)s - %(levelname)s - %(message)s",
+    level=logging.DEBUG,
+)
 
 
 class PipelineParameters(ParametersABC):
-    def __init__(self, general, tiler, baby, extraction, postprocessing):
+    def __init__(
+        self, general, tiler, baby, extraction, postprocessing, reporting=None
+    ):
         self.general = general
         self.tiler = tiler
         self.baby = baby
         self.extraction = extraction
         self.postprocessing = postprocessing
+        self.reporting = reporting
 
     @classmethod
     def default(
@@ -46,6 +59,7 @@ class PipelineParameters(ParametersABC):
         baby={},
         extraction={},
         postprocessing={},
+        reporting={},
     ):
         """
         Load unit test experiment
@@ -57,12 +71,14 @@ class PipelineParameters(ParametersABC):
         """
         expt_id = general.get("expt_id", 19993)
         directory = Path(general.get("directory", "../data"))
+        print("before dataset")
         with Dataset(int(expt_id), **general.get("server_info")) as conn:
             directory = directory / conn.unique_name
             if not directory.exists():
                 directory.mkdir(parents=True)
                 # Download logs to use for metadata
             conn.cache_logs(directory)
+        print("dataset exit")
         meta = MetaData(directory, None).load_logs()
         tps = meta["time_settings/ntimepoints"][0]
         defaults = {
@@ -70,10 +86,10 @@ class PipelineParameters(ParametersABC):
                 id=expt_id,
                 distributed=0,
                 tps=tps,
-                directory=directory,
+                directory=str(directory),
                 strain="",
                 earlystop=dict(
-                    min_tp=180,
+                    min_tp=100,
                     thresh_pos_clogged=0.3,
                     thresh_trap_clogged=7,
                     ntps_to_eval=5,
@@ -96,21 +112,17 @@ class PipelineParameters(ParametersABC):
 class Pipeline(ProcessABC):
     """
     A chained set of Pipeline elements connected through pipes.
+    Tiling, Segmentation,Extraction and Postprocessing should use their own default parameters.
+    These can be overriden passing the key:value of parameters to override to a PipelineParameters class
+
     """
 
-    # Tiling, Segmentation,Extraction and Postprocessing should use their own default parameters
-
-    # Early stop for clogging
-    earlystop = {
-        "min_tp": 180,
-        "thresh_pos_clogged": 0.3,
-        "thresh_trap_clogged": 7,
-        "ntps_to_eval": 5,
-    }
-
-    def __init__(self, parameters: PipelineParameters):
+    def __init__(self, parameters: PipelineParameters, store=None):
         super().__init__(parameters)
-        self.store = self.parameters.general["directory"]
+
+        if store is None:
+            store = self.parameters.general["directory"]
+        self.store = store
 
     @classmethod
     def from_yaml(cls, fpath):
@@ -118,11 +130,19 @@ class Pipeline(ProcessABC):
         # for other processes
         return cls(parameters=PipelineParameters.from_yaml(fpath))
 
+    @classmethod
+    def from_existing_h5(cls, fpath):
+        with h5py.File(fpath, "r") as f:
+            pipeline_parameters = PipelineParameters.from_yaml(f.attrs["parameters"])
+
+        return cls(pipeline_parameters)
+
     def run(self):
         # Config holds the general information, use in main
         # Steps holds the description of tasks with their parameters
         # Steps: all holds general tasks
         # steps: strain_name holds task for a given strain
+        print("run start")
         config = self.parameters.to_dict()
         expt_id = config["general"]["id"]
         distributed = config["general"]["distributed"]
@@ -172,28 +192,54 @@ class Pipeline(ProcessABC):
         earlystop = general_config["earlystop"]
         try:
             directory = general_config["directory"]
+
             with Image(image_id, **self.general["server_info"]) as image:
                 filename = f"{directory}/{image.name}.h5"
-                try:
-                    os.remove(filename)
-                except:
-                    pass
-
-                # Run metadata first
-                process_from = 0
-                # if True:  # not Path(filename).exists():
                 meta = MetaData(directory, filename)
-                meta.run()
-                tiler = Tiler.from_image(
-                    image, TilerParameters.from_dict(config["tiler"])
-                )
-                # else: TODO add support to continue local experiments?
-                #     tiler = Tiler.from_hdf5(image.data, filename)
-                #     s = Signal(filename)
-                #     process_from = s["/general/None/extraction/volume"].columns[-1]
-                #     if process_from > 2:
-                #         process_from = process_from - 3
-                #         tiler.n_processed = process_from
+                from_start = True
+                if (
+                    not general_config.get("overwrite", False)
+                    and Path(filename).exists()
+                ):
+                    try:
+                        print(f"Existing file {filename} will be used.")
+                        with h5py.File(filename, "r") as f:
+                            tiler = Tiler.from_hdf5(image.data, filename)
+                            s = Signal(filename)
+                            process_from = (
+                                f.attrs["last_processed"]
+                                or s.get_raw("/general/None/extraction/volume").columns[
+                                    -1
+                                ]
+                                or 0
+                            )
+                            # get state array
+                            state_array = f.get("state_array", 0)
+                        if process_from > 2:
+                            processFalsefrom = process_from - 3
+                            tiler.n_processed = process_from
+                        from_start = False
+                    except:
+                        pass
+
+                if from_start:  # New experiment or overwriting
+                    try:
+                        os.remove(filename)
+                    except:
+                        pass
+
+                    process_from = 0
+                    try:
+                        meta.run()
+                        meta.add_fields(
+                            {"omero_id,": config["general"]["id"], "image_id": image_id}
+                        )
+                        tiler = Tiler.from_image(
+                            image, TilerParameters.from_dict(config["tiler"])
+                        )
+                    except:
+                        # Remove and try to run again?
+                        pass
 
                 writer = TilerWriter(filename)
                 session = initialise_tf(2)
@@ -256,9 +302,15 @@ class Pipeline(ProcessABC):
                         t = perf_counter()
                         bwriter.write(seg, overwrite=["mother_assign"])
                         logging.debug(f"Timing:Writing-baby:{perf_counter() - t}s")
-                        t = perf_counter()
 
-                        tmp = ext.run(tps=[i])
+                        t = perf_counter()
+                        labels, masks = groupby_traps(
+                            seg["trap"],
+                            seg["cell_label"],
+                            seg["edgemasks"],
+                            tiler.n_traps,
+                        )
+                        tmp = ext.run(tps=[i], masks=masks, labels=labels)
                         logging.debug(f"Timing:Extraction:{perf_counter() - t}s")
                     else:  # Stop if more than X% traps are clogged
                         logging.debug(
@@ -276,13 +328,18 @@ class Pipeline(ProcessABC):
                         logging.debug(f"Quality:Clogged_traps:{frac_clogged_traps}")
                         print("Frac clogged traps: ", frac_clogged_traps)
 
+                    meta.add_fields({"last_processed": i})
                 # Run post processing
                 post_proc_params = PostProcessorParameters.from_dict(
                     self.parameters.postprocessing
                 ).to_dict()
                 PostProcessor(filename, post_proc_params).run()
-                return True
+
+                return 1
         except Exception as e:  # bug in the trap getting
+            logging.exception(
+                f"Caught exception in worker thread (x = {name}):", exc_info=True
+            )
             print(f"Caught exception in worker thread (x = {name}):")
             # This prints the type, value, and stack trace of the
             # current exception being handled.
@@ -293,6 +350,15 @@ class Pipeline(ProcessABC):
             if session:
                 session.close()
 
+        try:
+            compiler = ExperimentCompiler(None, filepath)
+            tmp = compiler.run()
+            po = PageOrganiser(tmp, grid_spec=(3, 2))
+            po.plot()
+            po.save(fullpath / f"{directory}report.pdf")
+        except Exception as e:
+            print(e)
+
     def check_earlystop(self, filename, es_parameters):
         s = Signal(filename)
         df = s["/extraction/general/None/area"]
@@ -306,3 +372,19 @@ class Pipeline(ProcessABC):
             > es_parameters["thresh_trap_clogged"]
         ).mean()
         return frac_clogged_traps
+
+
+def groupby_traps(traps, labels, edgemasks, ntraps):
+    # Group data by traps to pass onto extractor without re-reading hdf5
+    iterators = [
+        groupby(zip(traps, dset), lambda x: x[0]) for dset in (labels, edgemasks)
+    ]
+    label_d = {key: [x[1] for x in group] for key, group in iterators[0]}
+    mask_d = {
+        key: np.dstack([ndimage.morphology.binary_fill_holes(x[1]) for x in group])
+        for key, group in iterators[1]
+    }
+    labels = {i: label_d.get(i, []) for i in range(ntraps)}
+    masks = {i: mask_d.get(i, []) for i in range(ntraps)}
+
+    return labels, masks
